/**
* Filename: SerializationBuffer.java (in org.openbandy.io)
* This is part of the OpenBandy project.
* 
* www.openbandy.org
*/

package org.openbandy.io;

import java.util.Vector;

/**
* The SerializationBuffer is a temporary reservoir for
* the attributes of every serializable object that is
* serialized/deserialized. Thus, it holds a collection
* of ValueContainers.
* Moreover, it must hold every stateful variables that
* is used during the serialization/deserialization
* process.
*
* <br><br>
* (c) Copyright P. Bolliger 2007, ALL RIGHTS RESERVED.
*
* @author Philipp Bolliger (philipp@bolliger.name)
* @version 1.0
*/
public class SerializationBuffer {
	
	/* The list of value containers */
	protected Vector valueContainers;
	
	/* Shortcut to the current ValueContainer */
	protected ValueContainer currentValueContainer = null;
	
	/* Constructor */
	public SerializationBuffer(){
		valueContainers = new Vector();
	}
	
	/* ***************** Getter and Setter Methods **************** */
	
	/**
	 * Returns the vector containing all value container objects.
	 * Might return null if the serialization buffer has not been
	 * initialized.
	 * 
	 * @return Vector of all ValueContainer objects.
	 */
	public Vector getValueContainers(){
		return valueContainers;
	}
	
	/**
	 * Returns the current container object. If no container
	 * exists, the method returns null.
	 * 
	 * @return The current ValueContainer.
	 */
	public ValueContainer container(){
		return currentValueContainer;
	}
	
	/**
	 * Add a new container to the buffer.container(). After
	 * adding, this will be the current buffer.container().
	 * 
	 * @param container New buffer.container().
	 */
	public void addContainer(ValueContainer container){
		valueContainers.addElement(container);
		this.currentValueContainer = container;
	}
	
	/**
	 * Removes the given container from the list. If
	 * it was the currentContainer (i.e. the last added),
	 * the previous last is taken as current.
	 * 
	 * @param container The container that will be removed.
	 */
	public void removeContainer(ValueContainer container) {
		valueContainers.removeElement(container);
		if(!valueContainers.isEmpty()) {
			this.currentValueContainer = (ValueContainer)valueContainers.lastElement();
		}else {
			this.currentValueContainer = null;
		}
	}

}
