/**
* Filename: ObjectIdString.java (in org.openbandy.io.storage)
* This is part of the OpenBandy project.

* @author Philipp Bolliger (philipp@bolliger.name)
* @version 1.0
* 
* (c) Copyright 2007, ALL RIGHTS RESERVED.
*
* String representation of multiple object ids. With the method getNext()
* the numerical ids will be returned in the reverse order of insertion.
*/

package org.openbandy.io.storage;

import org.openbandy.io.SerializationIdentifiers;

public class ObjectIdString {
    
    private String objectIds;
    
    private final static String ID_DELIMITER = ";";
    
    public ObjectIdString(String initialString){
        this.objectIds = initialString;
    }
    
    public ObjectIdString(){
        objectIds = "";
    }
    
    public String getString(){
        return objectIds;
    }
    
    public void add(int id){
        objectIds = objectIds+id+ID_DELIMITER;
    }
    
    public int getNext(){
        if(objectIds != ""){
            
            /* extract first id */
            int firstDelimiter = objectIds.indexOf(ID_DELIMITER);
            int resultId = Integer.parseInt(objectIds.substring(0, firstDelimiter));
            
            /* remove first id from string */
            objectIds = objectIds.substring(firstDelimiter+1);
            
            return resultId;
        }else{
            return SerializationIdentifiers.NO_ID;
        }
    }

}
