package org.openbandy.log;

import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.rms.InvalidRecordIDException;
import javax.microedition.rms.RecordEnumeration;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;
import javax.microedition.rms.RecordStoreNotOpenException;

import org.openbandy.ui.ProgressAlert;


/**
 * TODO describe purpose and usage
 * 
 * can not (and don't have to) use RmsSerializer as this already makes intensiv use of log
 * 
 * @version 1.0
 * @author Philipp Bolliger (philipp@bolliger.name)
 */
public class LogStorage {

	/** The name of the record store holding the persistent log */
	private static final String RECORD_STORE_NAME = "LogMessages";

	/** Reference to the log canvas */
	private LogImpl logCanvas;

	/** The record store where messages are persistently stored */
	private RecordStore logRecordStore;


	/* *** Constructor *** */
	/**
	 * Create a new LogStorage.
	 * 
	 * @param logCanvas Reference to the log canvas
	 */
	public LogStorage(LogImpl logCanvas){
		this.logCanvas = logCanvas;

		try {
			/* open the record store where log message are stored */
			logRecordStore = RecordStore.openRecordStore(RECORD_STORE_NAME, true);
		}
		catch (Exception e) {
			logCanvas.error(this, e.getMessage(), e);
		}
	}


	/* ************** Methods of LogStorage *************** */

	/**
	 * Store a log message with any given log level to the RMS.
	 * 
	 * @param logMessage The message to store
	 * @return True if the messages was stored in the RMS
	 */
	protected void storeLogMessage(String logMessage) {
		try {
			/* store the log message in a new record */
			byte bytes[] = logMessage.getBytes();
			logRecordStore.addRecord(bytes,0,bytes.length);
		}
		catch (Exception e) {
			if(logCanvas != null){
				logCanvas.error(this, e.getMessage(), e);
			}
		}
	}

	/**
	 * Delete all persistentyl stored log messages.
	 */
	protected void clearStoredMessages(){
		try {
			/* close and delete log messages record store */
			logRecordStore.closeRecordStore();
			RecordStore.deleteRecordStore(LogStorage.RECORD_STORE_NAME);

			/* reopen */
			logRecordStore = RecordStore.openRecordStore(RECORD_STORE_NAME, true);

			if(logCanvas != null){
				logCanvas.info(this, "Cleared persistent log messages");
			}
		}
		catch (Exception e) {
			if(logCanvas != null){
				logCanvas.error(this, e.getMessage(), e);
			}
		}
	}

	/**
	 * Print all stored log messages to the associated log canvas.
	 */
	protected void displayStoredLogMessagesOnCanvas(){
		try {
			//OPTIMIZE only load the last XX entries, XX = LogConfiguration.MAX_ENTRIES
			/* read to log messages */
			RecordEnumeration re = logRecordStore.enumerateRecords(null, null, false);
			while(re.hasNextElement()){
				byte[] nextRec = re.nextRecord();

				/* display log messages on canvas */
				logCanvas.logOnCanvas(new String(nextRec));
			}
		}
		catch (Exception e) {
			logCanvas.error(this, e.getMessage(), e);
		}
	}

	/**
	 * Return String of all log messages.. TODO was genau, welches format, zusammenhang Log.getLogAsString... beschreiben!!
	 * 
	 * Because this can take very long time, show a progress alert to keep
	 * the user informed about the progress of this method.
	 */
	public String getPersistenLogAsString(Display display, Displayable currentDisplayable){		
		String logStorageString = "";
		try {
			int numRecords = logRecordStore.getNumRecords();
			if(numRecords > 0){
				/* create progress alert and set display */
				ProgressAlert progressAlert = new ProgressAlert("Reading "+numRecords+" log entries from storage..", AlertType.INFO, numRecords);
				progressAlert.setDisplay(display, currentDisplayable);

				progressAlert.show();

				/* read to log messages */
				RecordEnumeration re = logRecordStore.enumerateRecords(null, null, false);
				while(re.hasNextElement()){
					byte[] nextRec = re.nextRecord();

					String logMessage = new String(nextRec);
					int logLevel = Integer.parseInt(logMessage.substring(0,1)); //OPTIMIZE instead of parsing the log level of the message, keep it in separate data structure... (array[logIndex] )..
					logStorageString = logStorageString+LogLevel.levels[logLevel]+" "+logMessage.substring(1)+"\n";

					progressAlert.advance();
				}
				progressAlert.dismiss();
			}

		} catch (NumberFormatException nfe) {
			logCanvas.error(this, "NumberFormatException: "+nfe.getMessage(), nfe);
		} catch (RecordStoreNotOpenException rsnoe) {
			logCanvas.error(this, "RecordStoreNotOpenException: "+rsnoe.getMessage(), rsnoe);
		} catch (InvalidRecordIDException orie) {
			logCanvas.error(this, "InvalidRecordIDException: "+orie.getMessage(), orie);
		} catch (RecordStoreException rse) {
			logCanvas.error(this, "RecordStoreException: "+rse.getMessage(), rse);
		}
		return logStorageString;
	}

}
