/**
 * Filename: MessageBrokerClientImpl.java (in org.openbandy.net.client)
 * This file is part of the BandyBasics project.
 *
 * www.openbandy.org
 */

package org.openbandy.net.client;

import org.openbandy.io.Message;
import org.openbandy.net.ConnectionHelper;
import org.openbandy.net.MessageBroker;
import org.openbandy.net.ServerProtocol;
import org.openbandy.service.LogService;


/**
 * TODO describe purpose and usage
 * 
 *  NOTE: does not use MMS at the moment because Nokia 6630 and 6680
 *  J2ME implementation does not support MMS sending right now.
 *
 * <br><br>
 * (c) Copyright Philipp Bolliger 2007, ALL RIGHTS RESERVED.
 *
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 1.6
 * 
 */
public class MessageBrokerImpl implements MessageBroker {

	/* network connection to server */
	protected static ServerConnection networkConnection;

	/* queue for messages received from the server */
	protected static IncomingMessageQueue incomingMessageQueue;

//	/* maximum number of bytes that can be transmitted using one sms */
//	private final static int PAYLOAD_1_SMS = 133;
//	/* maximum number of bytes that can be transmitted using two sms */
//	public final static int PAYLOAD_2_SMS = 254;

//	/* static SMS sender */
//	private static SmsSender smsSender = null;

	/* static MMS sender */
//	private static MmsSender mmsSender = null;


	/* Constructor */
	public MessageBrokerImpl(){
		networkConnection = new ServerConnection();
		incomingMessageQueue = new IncomingMessageQueue();
	}


	/* ************ Methods for MessageBroker *************** */ 

	public synchronized Message sendMessage(String host, ServerProtocol protocol, Message message){
		/* connect to host */
		if(networkConnection.connect(host, protocol.getPort())){
			
			/* send message to host */
			ConnectionHelper.writeMessage(message, networkConnection.getOutputStream());
			
			/* read response and return if provided */
			Message response = ConnectionHelper.readMessage(networkConnection.getInputStream());
			if(response != null){
			
				/* check message validity */
				if(protocol.isValidMessage(response)){
					return response;
				}
				else{
					LogService.warn(this, "Response is not valid");
					return null;
				}
			}
		}else{
			LogService.warn(this, "No connection. Could not send message.");
		}
		return null;
	}

	//TODO impl & test
//	public synchronized void sendMessage(String cellPhoneNumber, Message message) {
//	GatewaySender gatewaySender = new GatewaySender(cellPhoneNumber, message);
//	gatewaySender.start();
//	}

	//TODO impl & test
//	public void sendEmail(String from, String to, String subject, String message) {
//	EmailSender emailSender = new EmailSender(from, to , subject, message);
//	emailSender.start();
//	}


	//TODO impl & test
//	/**
//	* Serializes the provided Message and writes it to the output stream.
//	* The output stream will NOT be closed by this method.
//	* 
//	* @param outputStream The output stream to write to.
//	* @param message The message to send.
//	*/
//	public synchronized void sendMessage(OutputStream outputStream, Message message){

//	byte[] buffer = SerializerService.getXmlStream(message, StreamSerializer.COMPRESSED_BEST).toByteArray();   
//	if(Assertion.assertNotNull(outputStream)){ 
//	try{
//	outputStream.write(buffer);
//	LogService.debug("MessageBrokerServerImpl", "Sent message: "+message.getLegibleString());
//	}catch(IOException ioe){
//	LogService.error("MessageBrokerServerImpl", ioe.getMessage(), ioe);;
//	}
//	}else{
//	LogService.warn("MessageBrokerServerImpl", "Could not send message");
//	}
//	}

	/* ********** Helper Classes and Methods ******************* */

//	private class EmailSender extends Thread{
//
//		private String from;
//		private String to;
//		private String subject;
//		private String message;
//
//		protected EmailSender(String from, String to, String subject, String message){
//			this.from = from;
//			this.to = to;
//			this.subject = subject;
//			this.message = message;
//		}
//
//		public void run(){
//			MmsMailSender mmsMailSender = new MmsMailSender();
//			mmsMailSender.sendMail(from, to, subject, message);
//		}
//
//	}
//
//	private class ServerSender extends Thread{
//		private Message message;
//
//		protected ServerSender(Message message){
//			this.message = message;
//		}
//
//		public void run(){
//			send();
//		}
//
//		private void send() {
//			/* send message via the output stream and close the output stream thereafter */
//			OutputStream outputStream = networkConnection.getOutputStream();
//
//			/* serialize message */
//			byte[] buffer = serializer.getXmlStream(message, StreamSerializer.COMPRESSED_BEST).toByteArray();
//
//			/* write message to output stream */
//			if(Assertion.notNull(outputStream)){ 
//				try{
//					outputStream.write(buffer);
//					outputStream.write(-1);
//					outputStream.close();
//					LogService.debug(this, "Sent message of size : "+buffer.length+" :: "+message.getLegibleString());
//				}catch(IOException ioe){
//					LogService.error(this, ioe.getMessage(), ioe);;
//				}
//			}else{
//				LogService.warn(this, "Could not send message, output stream is null");
//			}
//		}
//	}
//
//	private class GatewaySender extends Thread{
//
//		private String cellPhoneNumber;
//		private Message message;
//
//		protected GatewaySender(String cellPhoneNumber, Message message){
//			this.cellPhoneNumber = cellPhoneNumber;
//			this.message = message;
//		}
//
//		public void run() {
//			super.run();
//			/* serialize message */
//
//			//TODO use best compression here
//			byte[] buffer = serializer.getXmlStream(message, StreamSerializer.COMPRESSED_TOKEN).toByteArray();
//
//			//TODO test/release or remove MmsSender
////			if(buffer.length <= PAYLOAD_2_SMS){
//			/* send message using sms */
//			if(smsSender == null){
//				smsSender = new SmsSender();
//			}
//			smsSender.sendMessage(cellPhoneNumber, buffer);
//
////			}else{
////			/* send message using mms */
////			if(mmsSender == null){
////			mmsSender = new MmsSender();
////			}
////			mmsSender.sendMessage(cellPhoneNumber, buffer);
////			}
//		}
//
//	}

}
