/**
* Filename: MmsMailSender.java (in org.openbandy.net.client)
* This file is part of the BandyBasics project.
*
* www.openbandy.org
*/

package org.openbandy.net.client;

import java.io.IOException;

import javax.microedition.io.Connector;
import javax.wireless.messaging.MessageConnection;
import javax.wireless.messaging.MessagePart;
import javax.wireless.messaging.MultipartMessage;

import org.openbandy.service.LogService;

/**
 * This class provides the sendMail method which sends email messages
 * using MMS.
 * 
 * TODO test
 * NOTE: Does NOT work with Nokia 6630 and 6680!! untested!!
 * 
 * <br><br>
 * (c) Copyright Philipp Bolliger 2007, ALL RIGHTS RESERVED.
 *
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 0.1
 */
public class MmsMailSender {

	/* Network connection to other clients */
	private MessageConnection clientConnection;

	/* MIME type used */
	public final static String BANDY_MESSAGE_MIME = "application/bandyMessage";

	/* name of the payload part */
	public final static String BANDY_MMS_PAYLOAD = "payload";

	public void sendMail(String from, String to, String subject, String message) {
		try{
			/* open connection to client */
			String address = "mms://"+to;
			clientConnection = (MessageConnection)Connector.open(address);

			/* create binary sms message */
			MultipartMessage multipartMessage = (MultipartMessage)clientConnection.newMessage(MessageConnection.MULTIPART_MESSAGE);

			/* compose mms message */
			multipartMessage.setSubject(subject);
			String mimeType = "text/plain";
			String encoding = "UTF-8";
			byte[] contents = message.getBytes(encoding);
			MessagePart bandyMessage = new MessagePart(contents,
					0,
					contents.length,
					mimeType,
					"id1",
					"contentLocation",
					encoding);

			multipartMessage.addMessagePart(bandyMessage);

			/* send mms */
			clientConnection.send(multipartMessage);
			clientConnection.close();

			LogService.info(this, "Sent MMS to "+address);
		}catch(SecurityException se){
			/* the user did not allow to send mms */
			LogService.warn(this, "Not authorized to open connection for mms messages");
		}
		catch(IOException ioe){
			LogService.error(this, "Could not send message", ioe);
		}
	}

}
