/**
* Filename: MmsSender.java (in org.openbandy.net.client)
* This file is part of the BandyBasics project.
*
* www.openbandy.org
*/

package org.openbandy.net.client;

import java.io.IOException;

import javax.microedition.io.Connector;
import javax.wireless.messaging.MessageConnection;
import javax.wireless.messaging.MessagePart;
import javax.wireless.messaging.MultipartMessage;

import org.openbandy.net.MessageBroker;
import org.openbandy.service.LogService;

/**
 * TODO describe purpose and usage
 * 
 * <br><br>
 * (c) Copyright Philipp Bolliger 2007, ALL RIGHTS RESERVED.
 *
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 0.1
 */
public class MmsSender {
    
    /* Network connection to other clients */
    private MessageConnection clientConnection;
    
    /* MIME type used */
    public final static String BANDY_MESSAGE_MIME = "application/bandyMessage";
    
    /* name of the payload part */
    public final static String BANDY_MMS_PAYLOAD = "payload";

    public void sendMessage(String cellPhoneNumber, byte[] message) {
        if(cellPhoneNumber != ""){
            try{
                /* open connection to client */
                String address = "mms://"+cellPhoneNumber+":"+MessageBroker.MMS_PORT;
                clientConnection = (MessageConnection)Connector.open(address);
                
                /* create binary sms message */
                MultipartMessage multipartMessage = (MultipartMessage)clientConnection.newMessage(MessageConnection.MULTIPART_MESSAGE);
               
                /* compose mms message */
                multipartMessage.setSubject("MessagBrokerMessage");
                MessagePart bandyMessage = new MessagePart(
                                            message,            /* the actual bandy message */
                                            BANDY_MESSAGE_MIME, /* mime type */
                                            BANDY_MMS_PAYLOAD,  /* part name */
                                            null,               /* no file name */
                                            null);              /* no encoding */
                
                multipartMessage.addMessagePart(bandyMessage);
                
                /* send mms */
                clientConnection.send(multipartMessage);
                clientConnection.close();
                
                LogService.info(this, "Sent MMS to "+address);
            }catch(SecurityException se){
                /* the user did not allow to send mms */
            	LogService.warn(this, "Not authorized to open connection for mms messages");
            }
            catch(IOException ioe){
            	LogService.error(this, "Could not send message", ioe);
            }
        }else{
        	LogService.warn(this, "Could not send message. No target phone number");
        }

    }

}
