/**
* Filename: MainMenu.java (in org.openbandy.util)
* This file is part of the BandyBasics project.
*
* www.openbandy.org
*/

package  org.openbandy.ui;

import java.util.Hashtable;

import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.List;
import javax.microedition.midlet.MIDlet;

import org.openbandy.service.LogService;


/**
 * This class represents the Main Menu of the a Bandy MIDlet. It contains a
 * hashtable of all the screens that make up an application.
 * 
 * <br><br>
 * (c) Copyright Philipp Bolliger 2007, ALL RIGHTS RESERVED.
 *
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 1.5
 */
public class MainMenu extends List implements CommandListener {
	
	/* The surrouding Midlet */
	private MIDlet midlet;
	
	/* The surrouding Midlets Display, protected static such that it
	 * can be accessed by other UI classes */
	public static Display display;

	/* Screens */
	private Hashtable screens = new Hashtable();

	/* Commands */
	private Command cmdShow;
	private Command cmdBackground;
	private Command cmdExit;

	public MainMenu(MIDlet midlet, Display display){
		super("Main Menu", List.IMPLICIT);
		this.midlet = midlet;
		MainMenu.display = display;

		cmdShow = new Command("Show", Command.SCREEN, 1);
		cmdBackground = new Command("Run in Background", Command.SCREEN, 2);
		cmdExit = new Command("Exit", Command.EXIT, 3);

		addCommand(cmdShow);
		addCommand(cmdBackground);
		addCommand(cmdExit);

		setSelectCommand(cmdShow);
		setCommandListener(this);
	}

	public void addScreen(BandyDisplayable displayable){
		/* set references on displayable to come back to main menu */
		displayable.setDisplay(display, this);
		
		/* add it to the screen */
		try {
			String title = ((Displayable)displayable).getTitle();
			if(title != null) {
				Integer index = new Integer(append(title, null));
				screens.put(index, displayable);
			}
			else {
				LogService.warn(this, "Displayable has no title");
			}
		}
		catch(ClassCastException cce){
			LogService.warn(this, "BandyDisplayable not subclass of javax.microedition.lcdui.Displayable");
		}
	}

	public void commandAction(Command c, Displayable d) {
		if(c == cmdShow){
			Integer index = new Integer(getSelectedIndex());
			BandyDisplayable nextScreen = (BandyDisplayable) screens.get(index);
			nextScreen.updateScreen();
			display.setCurrent((Displayable)nextScreen);
		}
		else if (c == cmdBackground){
			display.setCurrent(null);
		}
		else if (c == cmdExit){
			midlet.notifyDestroyed();
		}
	}

}
